<?LassoScript

	// Asynchronous Email
	//
	// Copyright 2008, LassoSoft, LLC.
	//
	// Place this file into LassoStartup in the Lasso Professional 8
	// application folder.  Lasso must be restarted before the new
	// option can be used.  This file was designed for Lasso 8.5.5.  The
	// option will be included in Lasso 8.5.6.
	//
	// This file adds an -Async option to the [Email_Send],
	// [Email_Queue], and [Email_Batch] tags. Using this option allows
	// the Lasso page to complete faster while the email is sent in an
	// asynchronous thread.  This will result in better site performance
	// for visitors.  The option does not appear to affect email
	// delivery time significantly, but may slow it down moderately on a
	// busy server.
	//
	// [Email_Send] and [Email_Queue] - If -Async is specified the tag
	// will format the email and return immediately.  The email will be
	// inserted into the queue in an asynchronous thread.
	//
	// [Email_Batch] - If -Async is specified then every email message
	// queued within the container will be stored in a page variable. 
	// The closing tag will queue all of those messages in a single
	// asynchronous thread. Note that the queue is not paused until the
	// asynchronous portion of the tag.
	//
	// The [Email_Result] tag will not return a value when the -Async option
	// is used.
	//

	Define_Tag: 'Send',
		-Priority='replace',
		-namespace = 'email_',
		-priority='replace',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout',
		-Optional='priority',
		-Optional='to',
		-Optional='cc',
		-Optional='bcc',
		-Optional='from',
		-Optional='replyto',
		-Optional='sender',
		-Optional='subject',
		-Optional='body',
		-Optional='html',
		-Optional='transferencoding',
		-Optional='contenttype',
		-Optional='characterset',
		-Optional='attachments',
		-Optional='extramimeheaders',
		-Optional='simpleform',
		-Optional='tokens',
		-Optional='merge',
		-Optional='date',
		-Optional='async';

		fail_if: ((local_defined: 'from') && (!#from->size)) || (!(local_defined: 'from')), -1, '[Email_Send] Requires a -From address.';
		fail_if: !(local_defined: 'subject'), -1, '[Email_Send] Requires a -Subject.';
		fail_if: !(((local_defined: 'to') && (#to->size > 0)) || ((local_defined: 'cc') && (#cc->size > 0)) || ( (local_defined: 'bcc') && (#bcc->size > 0))), -1, '[Email_Send] Requires a recipient.';

		local(
			'_params' = params,
			'_host' = string(local('host')),
			'_username' = string(local('username')),
			'_password' = string(local('password')),
			'_async' = (local_defined('async') && (#async !== false)),
		);

		if((!#_username->size) && (!#_password->size) && (#_host >> '||') && (#_host->split('||')->size == 3));
			#_username = #_host->split('||')->get(1);
			#_password = #_host->split('||')->get(2);
			#_host = #_host->split('||')->get(3);
		else(#_host >> '||');
			#_host = #_host->split('||')->last;
		/if;

		if: (local_defined: 'simpleform');
			#_params->(removeall: '-simpleform');
			#_params->(removeall: '-body');

			if: !(local_defined: 'body');
				local: '_body' = string;
			else;
				local: '_body' = #body + '\n\n';
			/if;

			iterate: action_params, (local: 'i');
				if: !#i->first->(beginswith: '-');
					#_body += #i->first ': ' #i->second '\n';
				/if;
			/iterate;

			#_body += '\nForm was submitted by: ' client_ip '\n';
			#_body += 'Form was submitted on: ' date->(format: '%-D %-r %Z') '\n';
		
			#_params->(removeall: '-body');
			#_params->(insert: -body = @#_body);
		/if;

		local: 'data' = @(\email_compose->(run: -params=#_params, -name='email_compose'));
		local: 'mime_from' = #data->from;
		local: 'mime_recipients' = #data->recipients;
		local: 'mime_text' = #data->data;

		fail_if: (#mime_text == ''), -1, '[Email_Send] The MIME encoding for the email message could not be generated.';
		fail_if: (#mime_recipients == ''), -1, '[Email_Send] Requires one or more valid email addresses for recipients.';
		fail_if: (#mime_from == ''), -1, '[Email_Send] Requires a -From parameter with the sender\'s email address.';

		if: ((local_defined: 'date') == false) || (#date->(isa: 'date') == false);
			local: 'date' = date;
		/if;

		if: ((local: 'priority') == 'immediate') || (local_defined: 'immediate');
			email_Immediate:
				-data=#mime_text,
				-from=#mime_from,
				-recipients=#mime_recipients,
				-host = #_host,
				-port = (local: 'port'),
				-timeout = (local: 'timeout'),
				-username = #_username,
				-password = #_password,
				-tokens = (local: 'tokens'),
				-merge = (local: 'merge');
		else;
			email_Queue:
				-data=#mime_text,
				-from=#mime_from,
				-recipients=#mime_recipients,
				-priority=(local: 'priority'),
				-host = #_host,
				-port = (local: 'port'),
				-timeout = (local: 'timeout'),
				-username = #_username,
				-password = #_password,
				-tokens = (local: 'tokens'),
				-merge = (local: 'merge'),
				-date = #date,
				-async = #_async;
		/if;
	/Define_Tag;

	Define_Tag: 'Queue', -Priority='replace',
		-Privileged,
		-namespace = 'email_',
		-priority='replace',
		-Optional='data',
		-Optional='recipients',
		-Optional='from',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout',
		-Optional='priority',
		-Optional='tokens',
		-Optional='merge',
		-Optional='date',
		-Optional='async';

		fail_if: !(local_defined: 'data'), -1, '[Email_Queue] Requires a -Data parameter containing a MIME encoded email message.';
	
		if: #data->(isa: 'compose');		
			local: '_data' = #data->data;
			if: !(local_defined: 'recipients');
				local: 'recipients' = #data->recipients;
			/if;
			if: !(local_defined: 'from');
				local: 'from' = #data->from;
			/if;
		else;
			local: '_data' = #data;
		/if;

		fail_if: ((local: 'recipients') == ''), -1, '[Email_Queue] Requires a -Recipients parameter with one or more email addresses.';
		fail_if: ((local: 'from') == ''), -1, '[Email_Queue] Requires a -From parameter with the sender\'s email address.';

		if: (local_defined: 'host');
			local: 'smtp_host' = (if_null: (local: 'host'), '');
			local: 'smtp_port' = (if_null: (local: 'port'), '');
			local: 'smtp_username' = (if_null: (local: 'username'), '');
			local: 'smtp_password' = (if_null: (local: 'password'), '');
			local: 'smtp_timeout' = (if_null: (local: 'timeout'), '');
		/if;

		local: 'queue_priority' = 2;
		if: (local_defined: 'priority') && (#priority->(isa: 'string'));
			if: #priority >> 'high';
				local: 'queue_priority' = 1;
			else; #priority >> 'low';
				local: 'queue_priority' = 3;
			/if;
		/if;
		
		if: ((local_defined: 'date') == false) || (#date->(isa: 'date') == false);
			local: 'date' = date;
		/if;
		
		local: 'id' = lasso_uniqueid;

		local: 'sql' = 'INSERT INTO smtp_queue \
				(id_site,mime_text,mime_recipients,mime_from,smtp_host,smtp_port,smtp_timeout,smtp_username,smtp_password,queue_date,queue_status,queue_error,queue_number,queue_priority,create_date,merge_tokens,merge_array) \
				VALUES (\ 
				\'' + #id + '\',
				X\'' + (encode_hex: #_data) + '\', \
				\'' + (encode_sql92: (if_false: #recipients->(isa: 'array'), (array: #recipients), #recipients)->(join: ',')) + '\', \
				\'' + (encode_sql92: #from) + '\', \
				\'' + (encode_sql92: (local: 'smtp_host')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_port')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_timeout')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_username')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_password')) + '\', \
				\'' + (encode_sql92: #date->(format: '%Q %T')) + '\', \
				\'queued\', \
				\'\', \
				0, \
				3, \
				\'' + (encode_sql92: date->(format: '%Q %T')) + '\', \
				X\'' + (encode_hex: (local: 'tokens')->serialize) + '\', \
				X\'' + (encode_hex: (local: 'merge')->serialize) + '\');';
		
		if: (var_defined: '__email_batch__');
			$__email_batch__->insert(#sql);
			var: '_email_result' = null;
		else: (local_defined('async') && (#async !== false));
			{
				inline:
					-log=$__startup_loglevel__,
					-database='site',
					-table='smtp_queue',
					-keyfield='id',
					-sql=params->first;
				/inline;
				if: ((Global: '__SMTP_Queue_Status__') !>> 'Abort') && ((Global: '__SMTP_Queue_Status__') !>> 'Pause');
					Global: '__SMTP_Queue_Status__' = 'Active';
				/if;
			}->asasync(-params=array(#sql));
			var: '_email_result' = null;
		else;
			inline:
				-log=$__startup_loglevel__,
				-database='site',
				-table='smtp_queue',
				-keyfield='id',
				-sql=#sql;
				if: error_code != 0;
					log_critical: 'Queue Error: ' error_msg;
				else;
					var: '_email_result' = #id;
				/if;
			/inline;

			if: ((Global: '__SMTP_Queue_Status__') !>> 'Abort') && ((Global: '__SMTP_Queue_Status__') !>> 'Pause');
				Global: '__SMTP_Queue_Status__' = 'Active';
			/if;
		/if;

	/Define_Tag;

	Define_Tag: 'Batch',
			-namespace='email_',
			-priority='replace',
			-Container,
			-optional='async',
			-description='Pauses the email queue while a batch of emails is queued for sending.';
		error_reset;
		Local: 'output' = '';
		if: (local_defined('async') && (#async !== false));
		
			var: '__email_batch__' = array;
			Protect;
				#output = Run_Children;
				Handle_Error;
					#output += '[Email_Batch] Caught error ' + error_code + ' ' + error_msg;
				/Handle_Error;
			/Protect;
			
			{
				if: ((Global: '__SMTP_Queue_Status__') !>> 'Abort') && ((Global: '__SMTP_Queue_Status__') !>> 'Pause');
					Global: '__SMTP_Queue_Status__' = 'Pause Batch';
				/if;
				iterate: params, (local: 'sql');
					inline:
						-log=$__startup_loglevel__,
						-database='site',
						-table='smtp_queue',
						-keyfield='id',
						-sql=#sql;
					/inline;
				/iterate;
				if: ((Global: '__SMTP_Queue_Status__') >> 'Batch');
					Global: '__SMTP_Queue_Status__' = 'Active';
				/if;
	
			}->asasync(-params=$__email_batch__);
			var_remove('__email_batch__');
			
		else;
		
			if: ((Global: '__SMTP_Queue_Status__') !>> 'Abort') && ((Global: '__SMTP_Queue_Status__') !>> 'Pause');
				Global: '__SMTP_Queue_Status__' = 'Pause Batch';
			/if;
			Protect;
				#output = Run_Children;
				Handle_Error;
					#output += '[Email_Batch] Caught error ' + error_code + ' ' + error_msg;
				/Handle_Error;
			/Protect;
			if: ((Global: '__SMTP_Queue_Status__') >> 'Batch');
				Global: '__SMTP_Queue_Status__' = 'Active';
			/if;
			
		/if;
		Return: @#Output;
	/Define_Tag;
	
?>
